function [mu, sigma, map] = correction_step(mu, sigma, z, map);
% Updates the belief, i.e., mu and sigma after observing landmarks,
% and augments the map with newly observed landmarks.
% The employed sensor model measures the range and bearing of a landmark
% mu: state vector containing robot pose and poses of landmarks obeserved so far.
% Current robot pose = mu(1:3)
% Note that the landmark poses in mu are stacked in the order by which they were observed
% sigma: the covariance matrix of the system.
% z: struct array containing the landmark observations.
% Each observation z(i) has an id z(i).id, a range z(i).range, and a bearing z(i).bearing
% The vector 'map' contains the ids of all landmarks observed so far by the robot in the order
% by which they were observed, NOT in ascending id order.

% For computing sigma
global scale;

% Number of measurements in this time step
m = size(z, 2);

% Measurement noise
Q = 0.01*eye(2);

for i = 1:m
	% Get the id of the landmark corresponding to the i-th observation
	landmarkId = z(i).id;

	% Get the actual measurement made
	z_actual = [z(i).range; z(i).bearing];

	% If the landmark is observed for the first time:
	if(isempty(find(map==landmarkId)))
	    % Add it to the map
	    map = [map; landmarkId];
	    % TODO: Initialize its pose according to the measurement and add it to mu

	    % TODO: Initialize its uncertainty and add it to sigma


	    continue;
	endif

	% Compute sigma points from the predicted mean and covariance
	sigma_points = compute_sigma_points(mu, sigma);

	% Compute lambda
	n = length(mu);
	num_sig = size(sigma_points,2);
	lambda = scale - n;

	% TODO: Compute z_points (2x2n+1), which consists of predicted measurements from all sigma points


	% TODO: Compute the weights of the sigma points and compute zm.
	% zm is the recovered expected measurement mean from z_points.
	% It will be a 2x1 vector [expected_range; expected_bearing].


	% TODO: Compute the innovation covariance matrix S (2x2).


	% TODO: Compute sigma_x_z (which is equivalent to sigma times the Jacobian H transposed in EKF).
	% sigma_x_z is an nx2 matrix, where n is the current dimensionality of mu


	% TODO: Compute the Kalman gain


	% TODO: Update mu and sigma

	% TODO: Normalize the robot heading mu(3)


endfor
end
